/*******************************************************************************
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 *******************************************************************************/
package org.ow2.orchestra.console.GWTConsole.client.ListInstance;

import com.google.gwt.core.client.GWT;
import com.google.gwt.event.shared.HandlerManager;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.HorizontalPanel;

import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementException;
import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementServicesAsync;
import org.ow2.orchestra.console.GWTConsole.client.Commons.ActionImage;
import org.ow2.orchestra.console.GWTConsole.client.Commons.IndexFocusPanel;
import org.ow2.orchestra.console.GWTConsole.client.Commons.RowLabel;
import org.ow2.orchestra.console.GWTConsole.client.Commons.SpecialGrid;
import org.ow2.orchestra.console.GWTConsole.client.Commons.ActionClass;
import org.ow2.orchestra.console.GWTConsole.client.Event.RefreshDoneEvent;
import org.ow2.orchestra.console.GWTConsole.client.Event.SuspendTimerEvent;
import org.ow2.orchestra.console.GWTConsole.client.Event.ViewRefreshEvent;
import org.ow2.orchestra.console.GWTConsole.client.ListInstanceActivity.PListInstanceActivity;
import org.ow2.orchestra.console.GWTConsole.client.ListProcess.ProcessGrid;
import org.ow2.orchestra.console.GWTConsole.client.Popup.PPopup;
import org.ow2.orchestra.console.GWTConsole.client.Resources.ImageResources;
import org.ow2.orchestra.console.GWTConsole.client.Tab.PTab;
import org.ow2.orchestra.console.GWTConsole.client.TabLayoutPanel.PTabLayoutPanel;



/**
 * Class InstanceGrid that represents a grid containing instances. 
 * The class stores the instances in its grid and in a list. 
 * 
 * @author Jean-Yves Goubet
 * 
 */  
public final class InstanceGrid extends SpecialGrid {  
  
  private List<MInstance> mInstanceList;
  
  private int idLimitSize = 0;
  private int nameLimitSize = 0;
  private String gridDebugIdString = "instanceListGridId";
  
  private OrchestraManagementServicesAsync orchestramanagement = null;

  private String processId = null;

  private PTabLayoutPanel presenterTabLayoutPanel = null;

  private HandlerManager eventBus = null;
  
  private static String []columnTitles = {"UUID","Process UUID", "Process Name" ,
    "State","Start Date","Last Update Date","Actions"};
  
  /**
   * The constructor of the class
   * 
   * @author Jean-Yves Goubet
   * 
   */
  public InstanceGrid() {
    //
    super(columnTitles, 0, true);
  }
  
  /**
   * This method is called to refresh instances' list displayed on the client's screen.
   * For now the method consists in refreshing the whole list each time it's called.
   * 
   * @author Jean-Yves Goubet
   * 
   */
  public void refreshCurrentList() {
System.out.println("InstanceGrid - getInstances processId = " + processId);    
    this.orchestramanagement.getInstances(this.processId, new AsyncCallback<List<MInstance>>() {
      public void onSuccess(List<MInstance> mInstanceList) {
        InstanceGrid.this.mInstanceList = mInstanceList;
        InstanceGrid.this.sortAndDisplayList();
        InstanceGrid.this.eventBus.fireEvent(new RefreshDoneEvent());
      }

      public void onFailure(Throwable caught) {
        GWT.log("InstanceGrid - refreshCurrentList: onFailure : " + caught.getMessage());
        if (caught instanceof OrchestraManagementException) {
          InstanceGrid.this.eventBus.fireEvent(new SuspendTimerEvent(true));
          PPopup.launch(PPopup.PopupType.ErrorPopup, caught.getMessage(), 
              ((OrchestraManagementException)caught).getDetails());
        } else {
          PPopup.launch(PPopup.PopupType.ErrorPopup, "Unable to access to Orchestra !", caught.getMessage());
        }
      }
    });
  } 
  
  
  /**
   * This method is used to add an instance in the instances list displayed.
   * 
   * @author Jean-Yves Goubet
   * @param instance : it's the instance to add to the grid
   * 
   */
  protected void addInstance(final MInstance instance) {
    // add data to the end of the grid
    int rowIndex = this.getRowCount();
    //instantiate labels
    RowLabel[] rowLabels = new RowLabel[columnTitles.length - 1];
    rowLabels[0] = formatLabel(instance.getInstanceId(), idLimitSize, rowIndex);
    rowLabels[1] = formatLabel(instance.getProcessId(), idLimitSize, rowIndex);
    rowLabels[2] = formatLabel(instance.getProcessName(), nameLimitSize, rowIndex);
    rowLabels[3] = new RowLabel(instance.getInstanceState(),rowIndex,false);
    rowLabels[4] = new RowLabel(InstanceGrid.dateFormatter(instance.getInstanceStartDate()),rowIndex,false);
    rowLabels[5] = new RowLabel(InstanceGrid.dateFormatter(instance.getInstanceLastUpdateDate()),rowIndex,false);
    //addHandler to labels (for the style) and add labels to grid
    this.addData(rowLabels);
    
    // add action to instance
    this.addActionsToInstance(rowIndex, instance.getInstanceState());
  }  
  

  /**
   * Add the different action icon in action column for a given row.
   * The actions are suspend, resume, exit and delete.
   * 
   * @author Mathieu Crassin
   * @param rowIndex : The index of the row where we add the actions.
   * @param state : String that represent the instance state. 
   */
  private void addActionsToInstance(final int rowIndex, String state) {
    ActionImage [] tab = new ActionImage[5];
    
    
    // activities
    tab[0] = new ActionImage(ImageResources.INSTANCE.activities(), 
        ImageResources.INSTANCE.activitiesDisabled(), rowIndex);
    tab[0].setTitle("activities");
    // suspend action
    tab[1] = new ActionImage(ImageResources.INSTANCE.suspend(), 
        ImageResources.INSTANCE.suspendDisabled(), rowIndex);
    tab[1].setTitle("suspend");
    // resume action 
    tab[2] = new ActionImage(ImageResources.INSTANCE.resume(), 
        ImageResources.INSTANCE.resumeDisabled(), rowIndex);
    tab[2].setTitle("resume");
    // exit action 
    tab[3] = new ActionImage(ImageResources.INSTANCE.exit(), 
        ImageResources.INSTANCE.exitDisabled(), rowIndex);
    tab[3].setTitle("exit");
    // delete action 
    tab[4] = new ActionImage(ImageResources.INSTANCE.deleteIcon(), 
        ImageResources.INSTANCE.deleteIconDisabled(), rowIndex);
    tab[4].setTitle("remove");
    
    this.addActions(tab, state, 6);
  }
  
  /**
   * Function call in the SpecialGrid class to attach some actions with each icon in action 
   * column.
   * 
   * @author Mathieu Crassin
   * @param tabImages : a tab that represent the different actions 
   * @param state : state of the information display on the row 
   */
  protected void addClickHandlers(final ActionImage[] tabImages, String state) {
    
    // activities
    tabImages[0].setHandler(new ActionClass() {
      public void execute() {
        String instanceId = InstanceGrid.this.getText(tabImages[0].getRowIndex(),0);
        
        // create the presenter of list process activity 
        new PListInstanceActivity(InstanceGrid.this.presenterTabLayoutPanel, instanceId,
            InstanceGrid.this.getText(tabImages[0].getRowIndex(),2) + " " + instanceId,  eventBus);
      }
    });
    
    // suspend 
    tabImages[1].setHandler(new ActionClass() {
      public void execute() {
        System.out.println("action suspend on " + tabImages[0].getRowIndex());
        actionSuspend(tabImages[0]);
      }
    });
    
    // resume
    tabImages[2].setHandler(new ActionClass() {
      public void execute() {
        System.out.println("action resume");
        actionResume(tabImages[0]);
      }
    });

    // exit
    tabImages[3].setHandler(new ActionClass() {
      public void execute() {
        System.out.println("action exit");
        actionExit(tabImages[0]);
      }
    });
    
    // delete
    tabImages[4].setHandler(new ActionClass() {
      public void execute() {
        System.out.println("action delete");
        actionRemove(tabImages[0]);
      }
    });    
    
    // define the state of each action (enable or disable)
    if (state.equals("finished")) {
      tabImages[1].setEnabled(false);
      tabImages[2].setEnabled(false);
      tabImages[3].setEnabled(false);
      tabImages[4].setEnabled(true);
    } else if (state.equals("running")) {
      tabImages[1].setEnabled(true);
      tabImages[2].setEnabled(false);
      tabImages[3].setEnabled(true);
      tabImages[4].setEnabled(false);
    } else if (state.equals("suspended")) {
      tabImages[1].setEnabled(false);
      tabImages[2].setEnabled(true);
      tabImages[3].setEnabled(true);
      tabImages[4].setEnabled(false);
    } else if (state.equals("exited")) {
      tabImages[1].setEnabled(false);
      tabImages[2].setEnabled(false);
      tabImages[3].setEnabled(false);
      tabImages[4].setEnabled(true);
    } else {
      System.out.println("state pb");
    }
  }

  /**
   * Function call by the handler on the suspend icon.
   * 
   * @author Mathieu Crassin
   * @param tab : represent the different actions possibles
   */
  protected void actionSuspend(final ActionImage actionImg) {
    RowLabel uuidlabel = (RowLabel) this.getWidget(actionImg.getRowIndex(), 0);
    orchestramanagement.suspendInstance(uuidlabel.getText(), new AsyncCallback<Void>() {
      
      public void onSuccess(Void arg0) {
        InstanceGrid.this.eventBus.fireEvent(new ViewRefreshEvent(InstanceGrid.this.processId, PTab.Type.LIST_INSTANCE, true));
      }
      
      public void onFailure(Throwable arg0) {
        InstanceGrid.this.eventBus.fireEvent(new SuspendTimerEvent(true));
        if (arg0 instanceof OrchestraManagementException) {
          PPopup.launch(PPopup.PopupType.ErrorPopup, arg0.getMessage(), ((OrchestraManagementException)arg0).getDetails());
        } else {
          PPopup.launch(PPopup.PopupType.ErrorPopup, "Fail while suspend instance !", arg0.getMessage());
        }        
      }
    });
  }
  
  /**
   * Function call by the handler on the resume icon.
   * 
   * @author Mathieu Crassin
   * @param tab : represent the different actions possibles
   */
  protected void actionResume(final ActionImage actionImg) {
    RowLabel uuidlabel = (RowLabel) this.getWidget(actionImg.getRowIndex(), 0);
    orchestramanagement.resumeInstance(uuidlabel.getText(), new AsyncCallback<Void>() {
      
      public void onSuccess(Void arg0) {
        InstanceGrid.this.eventBus.fireEvent(new ViewRefreshEvent(InstanceGrid.this.processId, PTab.Type.LIST_INSTANCE, true));
      }
      
      public void onFailure(Throwable arg0) {
        if (arg0 instanceof OrchestraManagementException) {
          PPopup.launch(PPopup.PopupType.ErrorPopup, arg0.getMessage(), ((OrchestraManagementException)arg0).getDetails());
        } else {
          PPopup.launch(PPopup.PopupType.ErrorPopup, "Fail while resume instance !", arg0.getMessage());
        }           
      }
    });
  }
  
  /**
   * Function call by the handler on the exit icon.
   * 
   * @author Mathieu Crassin
   * @param tab : represent the different actions possibles
   */  
  protected void actionExit(final ActionImage actionImg) {
    RowLabel uuidlabel = (RowLabel) this.getWidget(actionImg.getRowIndex(), 0);
    orchestramanagement.exitInstance(uuidlabel.getText(), new AsyncCallback<Void>() {
      
      public void onSuccess(Void arg0) {
        InstanceGrid.this.eventBus.fireEvent(new ViewRefreshEvent(InstanceGrid.this.processId, PTab.Type.LIST_INSTANCE, true));
      }
      
      public void onFailure(Throwable arg0) {
        if (arg0 instanceof OrchestraManagementException) {
          PPopup.launch(PPopup.PopupType.ErrorPopup, arg0.getMessage(), ((OrchestraManagementException)arg0).getDetails());
        } else {
          PPopup.launch(PPopup.PopupType.ErrorPopup, "Fail while exit instance !", arg0.getMessage());
        }   
      }
    });
  }
  
  /**
   * Function call by the handler on the delete icon.
   * 
   * @author Mathieu Crassin
   * @param tab : represent the different actions possibles
   */  
  protected void actionRemove(final ActionImage actionImg) {
    RowLabel uuidlabel = (RowLabel) this.getWidget(actionImg.getRowIndex(), 0);
    orchestramanagement.removeInstance(uuidlabel.getText(), new AsyncCallback<Void>() {
      
      public void onSuccess(Void arg0) {
        InstanceGrid.this.eventBus.fireEvent(new ViewRefreshEvent(InstanceGrid.this.processId, PTab.Type.LIST_INSTANCE, true));
        
        // delete instance activity tab if exists
        InstanceGrid.this.presenterTabLayoutPanel.removeTab(
            InstanceGrid.this.getText(actionImg.getRowIndex(), 0), 
            PTab.Type.LIST_INSTANCE_ACTIVITY);
      }
      
      public void onFailure(Throwable arg0) {
        if (arg0 instanceof OrchestraManagementException) {
          PPopup.launch(PPopup.PopupType.ErrorPopup, arg0.getMessage(), ((OrchestraManagementException)arg0).getDetails());
        } else {
          PPopup.launch(PPopup.PopupType.ErrorPopup, "Fail while remove instance !", arg0.getMessage());
        }   
      }
    });
  }  

  /**
   * This method is used to sort and display the instances' list depending on the column and order values
   * 
   * @author Jean-yves Goubet
   * 
   */
  public void sortAndDisplayList() {
    if (mInstanceList != null) {
      MInstanceCompare comparator = new MInstanceCompare(this.sortColumnIndex, this.order);
      Collections.sort(mInstanceList, comparator);
      this.manageGrid();
      for (MInstance m : mInstanceList) {
        addInstance(m);
      }
      this.setDebugId();
    }
  }  
  
  /**
   * This class is the comparator used to sort the list of instances
   * 
   * @author Jean-Yves Goubet
   */
  private class MInstanceCompare implements Comparator<MInstance> {
    private int compareType;
    private boolean order;
    
    public int compare(MInstance o1, MInstance o2) {
      switch (compareType) {
        //Instance UUID
        case 0: 
          return order ? Integer.valueOf(o1.getInstanceId().replace("$", "")).compareTo(Integer
              .valueOf(o2.getInstanceId().replace("$", ""))) : Integer.valueOf(o2.getInstanceId()
                  .replace("$", "")).compareTo(Integer.valueOf(o1.getInstanceId().replace("$", "")));
        //Process UUID
        case 1: 
          return order ? Integer.valueOf(o1.getProcessId()).compareTo(Integer.valueOf(o2.getProcessId())) 
              : Integer.valueOf(o2.getProcessId()).compareTo(Integer.valueOf(o1.getProcessId()));
        // Process Name
        case 2: 
          return order ? o1.getProcessName().compareTo(o2.getProcessName()) 
            : o2.getProcessName().compareTo(o1.getProcessName());          
        //Instance state
        case 3: 
          return order ? o1.getInstanceState().compareTo(o2.getInstanceState())
              : o2.getInstanceState().compareTo(o1.getInstanceState());
        //Start Date
        case 4: 
          return order ? o1.getInstanceStartDate().compareTo(o2.getInstanceStartDate())
            : o2.getInstanceStartDate().compareTo(o1.getInstanceStartDate());
        //End Date
        case 5:
          if ((o1.getInstanceState().equals("finished") || o1.getInstanceState().equals("exited"))
              && (o2.getInstanceState().equals("finished") || o2.getInstanceState().equals("exited"))) {
            return order ? o1.getInstanceLastUpdateDate().compareTo(o2.getInstanceLastUpdateDate())
                : o2.getInstanceLastUpdateDate().compareTo(o1.getInstanceLastUpdateDate());
          } else if (o1.getInstanceState().equals("finished") || o1.getInstanceState().equals("exited")) {
            return order ? 1 : -1;
          } else if (o2.getInstanceState().equals("finished") || o2.getInstanceState().equals("exited")) {
            return order ? -1 : 1;
          }
        //default case (return 0 as if the 2 elements to compare were equals)
        default: 
          return 0;
      }
    }
    
    public MInstanceCompare(int compareType, boolean order) {
      this.compareType = compareType;
      this.order = order;
    }
  }
  
  protected void setOrchestraManagementServices(OrchestraManagementServicesAsync orchestramanagement) {
    this.orchestramanagement = orchestramanagement;
  }    

  protected void setStringLimitSize(final int idLimitSize, final int nameLimitSize, final String exceededString) {
    this.idLimitSize = idLimitSize;
    this.nameLimitSize = nameLimitSize;
    this.exceededString = exceededString;
  }
  
  /**
   * Define the debug id of the grid.
   * 
   * @author Mathieu Crassin
   */
  protected void setDebugId() {
    this.ensureDebugId(gridDebugIdString);
  }
  /**
   * Function automatically call to define the debug id of the InstanceGrid class.
   * 
   * @author Mathieu Crassin
   * @param baseID : represent the base string of every ID for the instance grid.
   */
  protected void onEnsureDebugId(String baseID) {
    if (processId != null) {
      super.onEnsureDebugId(baseID + "-UUID-" + processId);
    } else {
      super.onEnsureDebugId(baseID);
    }

    int row = this.getRowCount();
    int col = this.getColumnCount();
    if (row > 1 && col == 7) {
      
      for (int i = 1; i < row; i++) {    
        ensureDebugId(((HorizontalPanel) ((IndexFocusPanel) this.getWidget(i, col - 1)).getWidget()).
            getWidget(0).getElement(), baseID + "-" + i + "-" + (col - 1), "activitiesImageID");
        ensureDebugId(((HorizontalPanel) ((IndexFocusPanel) this.getWidget(i, col - 1)).getWidget()).
            getWidget(1).getElement(), baseID + "-" + i + "-" + (col - 1), "suspendImageID");
        ensureDebugId(((HorizontalPanel) ((IndexFocusPanel) this.getWidget(i, col - 1)).getWidget()).
            getWidget(2).getElement(), baseID + "-" + i + "-" + (col - 1), "resumeImageID");
        ensureDebugId(((HorizontalPanel) ((IndexFocusPanel) this.getWidget(i, col - 1)).getWidget()).
            getWidget(3).getElement(), baseID + "-" + i + "-" + (col - 1), "exitImageID");
        ensureDebugId(((HorizontalPanel) ((IndexFocusPanel) this.getWidget(i, col - 1)).getWidget()).
            getWidget(4).getElement(), baseID + "-" + i + "-" + (col - 1), "removeImageID");        
      }  
    }
  }

  /**
   * Set method for process ID object.
   * This process ID is used by the refresh function (to get the instance list attach to the 
   * process)  
   * 
   * @author Mathieu Crassin
   * @param processId
   */
  public void setProcessId(String processId) {
    this.processId = processId;
  }
  
  /**
   * Allow the process grid class to access to the tab organization.
   * 
   * @param tabLayoutPanel
   */
  public void setPresenterTabLayoutPanel(PTabLayoutPanel presenterTabLayoutPanel) {
    this.presenterTabLayoutPanel  = presenterTabLayoutPanel;
  }

  /**
   * 
   * @param eventBus
   */
  public void setHandlerManager(HandlerManager eventBus) {
    this.eventBus  = eventBus;
  }  
}

