/*******************************************************************************
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 *******************************************************************************/
package org.ow2.orchestra.console.GWTConsole.client.ListInstanceActivity;

import com.google.gwt.core.client.GWT;
import com.google.gwt.event.shared.HandlerManager;
import com.google.gwt.user.client.rpc.AsyncCallback;

import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementException;
import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementServices;
import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementServicesAsync;
import org.ow2.orchestra.console.GWTConsole.client.Commons.ActionImage;
import org.ow2.orchestra.console.GWTConsole.client.Commons.RowLabel;
import org.ow2.orchestra.console.GWTConsole.client.Commons.SpecialGrid;
import org.ow2.orchestra.console.GWTConsole.client.Event.RefreshDoneEvent;
import org.ow2.orchestra.console.GWTConsole.client.Event.SuspendTimerEvent;
import org.ow2.orchestra.console.GWTConsole.client.Popup.PPopup;


/**
 * Class ActivityGrid that represents a grid containing activities for instances. 
 * The class stores the activities in its grid and in a list
 * 
 * @author Jean-Yves Goubet
 * 
 */  
public class InstanceActivityGrid extends SpecialGrid {

  protected List<MInstanceActivity> mInstanceActivityList;
  
  private int nameLimitSize = 0;
  private int typeLimitSize = 0;
  
  private final OrchestraManagementServicesAsync orchestramanagement = GWT.create(OrchestraManagementServices.class);
  
  private final String gridDebugIdString = "instanceActivityListGridId";
  
  private String instanceId = "";

  protected HandlerManager eventBus = null;
  
  /**
   * The constructor of the class
   * 
   * @author Jean-Yves Goubet
   * 
   */
  public InstanceActivityGrid() {
    //
    super();
    //
    String []columnTitles = {"Name", "Type", "State", "Start Date", "End Date"};
    //
    initGrid(columnTitles, 0);
  }
  
  public void setInstanceId(String instanceId) {
    this.instanceId = instanceId;
  }
  
  public void setStringLimits(final int nameLimitSize, final int typeLimitSize, final String exceededString) {
    this.nameLimitSize = nameLimitSize;
    this.typeLimitSize = typeLimitSize;
    this.exceededString = exceededString;
  }
  
  /**
   * This method is used to add an activity in the list displayed
   * 
   * @author Jean-Yves Goubet
   * @param instanceActivity : it's the activity to add to the grid
   * 
   */
  protected void addInstanceActivity(final MInstanceActivity instanceActivity) {
    // add data to the end of the grid
    int rowIndex = this.getRowCount();
    //instantiate labels
    RowLabel[] rowLabels = new RowLabel[columnTitles.length];
    rowLabels[0] = new RowLabel(instanceActivity.getInstanceActivityName(), rowIndex, false);
    rowLabels[1] = new RowLabel(instanceActivity.getInstanceActivityType(), rowIndex, false); 
    rowLabels[2] = new RowLabel(instanceActivity.getInstanceActivityState(), rowIndex, false);
    rowLabels[3] = new RowLabel(dateFormatter(instanceActivity.getInstanceActivityBeginDate()), rowIndex, false);
    rowLabels[4] = new RowLabel(dateFormatter(instanceActivity.getInstanceActivityEndDate()), rowIndex, false);  
    //add to grid
    this.addData(rowLabels);
    // fix the height of cells
    for (int i = 0;i < columnTitles.length;i++) {
      this.getCellFormatter().setHeight(rowIndex, i, "23px");
    }
  }  
  
  
  /**
   * This method is called to refresh instance's activities list displayed on the client's screen.
   * 
   * @author Jean-Yves Goubet
   * 
   */
  @Override
  public void refreshCurrentList() {
    this.orchestramanagement.getInstanceActivities(this.instanceId, new AsyncCallback<List<MInstanceActivity>>() {
      public void onSuccess(List<MInstanceActivity> mInstanceActivityList) {
        InstanceActivityGrid.this.mInstanceActivityList = mInstanceActivityList;
        InstanceActivityGrid.this.sortAndDisplayList();     
        InstanceActivityGrid.this.eventBus.fireEvent(new RefreshDoneEvent());
      }

      public void onFailure(Throwable caught) {
        GWT.log("InstanceActivityGrid - refreshCurrentList: onFailure : " + caught.getMessage());
        InstanceActivityGrid.this.eventBus.fireEvent(new SuspendTimerEvent(true));
        if (caught instanceof OrchestraManagementException) {
          PPopup.launch(PPopup.PopupType.ErrorPopup, caught.getMessage(), 
              ((OrchestraManagementException)caught).getDetails());
        } else {
          PPopup.launch(PPopup.PopupType.ErrorPopup, "Unable to access to Orchestra !", caught.getMessage());
        }
      }
    });
  }
  
  
  /**
   * This method consists in sorting the list and display it on client's screen
   * 
   * @author Jean-Yves Goubet
   */
  @Override
  public void sortAndDisplayList() {
    if (mInstanceActivityList != null) {
      MInstanceActivityCompare comparator = new MInstanceActivityCompare(this.sortColumnIndex, this.order);
      Collections.sort(mInstanceActivityList, comparator);
      this.manageGrid();
      for (MInstanceActivity m : mInstanceActivityList) {
        addInstanceActivity(m);
      }
      this.setDebugId();
    }
  }
  
  
  /**
   * This method sets ids on the grid's elements.
   * 
   * @author Jean-Yves Goubet
   */
  private void setDebugId() {
    this.ensureDebugId(gridDebugIdString + "-UUID-" + instanceId + "-IA");    
  }


  /**
   * This class is the comparator used to sort the list of instance's activities
   * 
   * @author Jean-Yves Goubet
   */
  private class MInstanceActivityCompare implements Comparator<MInstanceActivity> {
    private final int compareType;
    private final boolean order;
    
    public int compare(MInstanceActivity o1, MInstanceActivity o2) {
      switch (compareType) {
        // Activity Name
        case 0:
          return order ? o1.getInstanceActivityName().compareTo(o2.getInstanceActivityName()) : o2.getInstanceActivityName()
              .compareTo(o1.getInstanceActivityName());
        // Activity Type
        case 1: 
          return order ? o1.getInstanceActivityType().compareTo(o2.getInstanceActivityType()) 
              : o2.getInstanceActivityType().compareTo(o1.getInstanceActivityType());
        // Activity State
        case 2: 
          return order ? o1.getInstanceActivityState().compareTo(o2.getInstanceActivityState()) 
            : o2.getInstanceActivityState().compareTo(o1.getInstanceActivityState());          
        // Activity Begin Date
        case 3: 
          return order ? o1.getInstanceActivityBeginDate().compareTo(o2.getInstanceActivityBeginDate())
              : o2.getInstanceActivityBeginDate().compareTo(o1.getInstanceActivityBeginDate());
        // Activity End Date
        case 4: 
          if ((o1.getInstanceActivityState().equals("finished") || o1.getInstanceActivityState().equals("exited") || o1.getInstanceActivityState().equals("terminated"))
              && (o2.getInstanceActivityState().equals("finished") || o2.getInstanceActivityState().equals("exited") || o2.getInstanceActivityState().equals("terminated"))) {
            return order ? o1.getInstanceActivityEndDate().compareTo(o2.getInstanceActivityEndDate())
                : o2.getInstanceActivityEndDate().compareTo(o1.getInstanceActivityEndDate());
          } else if ((o1.getInstanceActivityState().equals("finished") || o1.getInstanceActivityState().equals("exited") || o1.getInstanceActivityState().equals("terminated"))) {
            return order ? 1 : -1;
          } else if ((o2.getInstanceActivityState().equals("finished") || o2.getInstanceActivityState().equals("exited") || o2.getInstanceActivityState().equals("terminated"))) {
            return order ? -1 : 1;
          }
        // default case (return 0 as if the 2 elements to compare were equals)
        default: 
          return 0;
      }
    }
    
    public MInstanceActivityCompare(int compareType, boolean order) {
      this.compareType = compareType;
      this.order = order;
    }
  }


  public void setHandlerManager(HandlerManager eventBus) {
    this.eventBus = eventBus;
  }

  /**
   * This function isn't use in this class because there aren't any actions column.
   * 
   * @author Mathieu Crassin
   */
  @Override
  protected void addClickHandlers(ActionImage[] tabImages, String state) {
  }
}
