/*******************************************************************************
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 *******************************************************************************/
package org.ow2.orchestra.console.GWTConsole.client.ListProcessActivity;

import com.google.gwt.core.client.GWT;
import com.google.gwt.event.shared.HandlerManager;
import com.google.gwt.user.client.rpc.AsyncCallback;

import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementException;
import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementServices;
import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementServicesAsync;
import org.ow2.orchestra.console.GWTConsole.client.Commons.ActionImage;
import org.ow2.orchestra.console.GWTConsole.client.Commons.RowLabel;
import org.ow2.orchestra.console.GWTConsole.client.Commons.SpecialGrid;
import org.ow2.orchestra.console.GWTConsole.client.Event.RefreshDoneEvent;
import org.ow2.orchestra.console.GWTConsole.client.Event.SuspendTimerEvent;
import org.ow2.orchestra.console.GWTConsole.client.Popup.PPopup;


/**
 * Class ProcessActivityGrid that represents a grid containing activities for process. 
 * The class stores the activities in its grid and in a list
 * 
 * @author Jean-Yves Goubet
 * 
 */  
public class ProcessActivityGrid extends SpecialGrid {

  protected List<MProcessActivity> mProcessActivityList = null;
  
  private String gridDebugIdString = "processActivityListGridId";
  
  private int idLimitSize = 0;
  private int typeLimitSize = 0;
  private int nameLimitSize = 0;
  
  private String processId = "";
  
  private OrchestraManagementServicesAsync orchestramanagement = GWT.create(OrchestraManagementServices.class);

  private HandlerManager eventBus = null;
  
  /**
   * The constructor of the class
   * 
   * @author Jean-Yves Goubet
   * 
   */
  public ProcessActivityGrid() {
    //
    super();

    // init the grid by calling special grid constructor
    String []columnTitles = {"Name","Type"};
    initGrid(columnTitles, 0);
  }
  
  public void setProcessId(String processId) {
    this.processId = processId;
  }
  
  /**
   * This method gets the activity of a given process from Orchestra through the orchestra servlet.
   * 
   * @author Jean-Yves Goubet
   * @param processID : the process UUID to get activities
   */
  @Override
  public void refreshCurrentList() {
//System.out.println("ProcessActivityGrid - processId = " + processId);
    
    this.orchestramanagement.getProcessActivities(processId, new AsyncCallback<List<MProcessActivity>>() {
      public void onSuccess(List<MProcessActivity> mProcessActivityList) {
        ProcessActivityGrid.this.mProcessActivityList = mProcessActivityList;
        ProcessActivityGrid.this.sortAndDisplayList();
        ProcessActivityGrid.this.eventBus.fireEvent(new RefreshDoneEvent());
      }

      public void onFailure(Throwable caught) {
        GWT.log("ProcessActivityGrid - getProcessActivities: onFailure : " + caught.getMessage());
        ProcessActivityGrid.this.eventBus.fireEvent(new SuspendTimerEvent(true));
        if (caught instanceof OrchestraManagementException) {
          PPopup.launch(PPopup.PopupType.ErrorPopup, caught.getMessage(), 
              ((OrchestraManagementException)caught).getDetails());
        } else {
          PPopup.launch(PPopup.PopupType.ErrorPopup, "Unable to access to Orchestra !", caught.getMessage());
        }
      }
    });
  }

  /**
   * This method is used to sort and display the process activities list
   * 
   * @author Jean-Yves Goubet
   * 
   */
  @Override
  public void sortAndDisplayList() {
    if (mProcessActivityList != null) {
      MProcessActivityCompare comparator = new MProcessActivityCompare(this.sortColumnIndex, this.order);
      Collections.sort(mProcessActivityList, comparator);
      this.manageGrid();
      for (MProcessActivity m : mProcessActivityList) {
        addProcessActivity(m);
      }
      this.setDebugId();
    }
  }

  
  /**
   * This method sets ids to grid elements
   * 
   * @author Jean-Yves Goubet
   */
  private void setDebugId() {
    this.ensureDebugId(gridDebugIdString + "-UUID-" + processId + "-PA");
  }


  /**
   * This method is used to add an activity in the list displayed
   * 
   * @author Jean-Yves Goubet
   * @param processActivity : it's the activity to add to the grid
   * 
   */
  protected void addProcessActivity(final MProcessActivity processActivity) {
    // add data to the end of the grid
    int rowIndex = this.getRowCount();
    //instantiate labels
    RowLabel[] rowLabels = new RowLabel[columnTitles.length];
    rowLabels[0] = new RowLabel(processActivity.getProcessActivityName(), rowIndex, false);
    rowLabels[1] = new RowLabel(processActivity.getProcessActivityType(), rowIndex, false); 
    //add to grid
    this.addData(rowLabels);
    // fix the height of cells
    for (int i = 0;i < columnTitles.length;i++) {
      this.getCellFormatter().setHeight(rowIndex, i, "23px");
    }
  }
  
  
  /**
   * This class is the comparator used to sort the list of process' activities
   * 
   * @author Jean-Yves Goubet
   */
  private class MProcessActivityCompare implements Comparator<MProcessActivity> {
    private int compareType;
    private boolean order;
    
    public int compare(MProcessActivity o1, MProcessActivity o2) {
      switch (compareType) {
        // Activity Name
        case 0:
          return order ? o1.getProcessActivityName().compareTo(o2.getProcessActivityName()) : o2.getProcessActivityName()
              .compareTo(o1.getProcessActivityName());
        // Activity Type
        case 1: 
          return order ? o1.getProcessActivityType().compareTo(o2.getProcessActivityType()) 
              : o2.getProcessActivityType().compareTo(o1.getProcessActivityType());
        //default case (return 0 as if the 2 elements to compare were equals)
        default: 
          return 0;
      }
    }
    
    public MProcessActivityCompare(int compareType, boolean order) {
      this.compareType = compareType;
      this.order = order;
    }
  }


  public void setHandlerManager(HandlerManager eventBus) {
    this.eventBus = eventBus;
  }

  /**
   * This function isn't use in this class because there aren't any actions column.
   * 
   * @author Mathieu Crassin
   */
  @Override
  protected void addClickHandlers(ActionImage[] tabImages, String state) {
  }
}
